@@ -35,7 +35,8 @@ C_MainWindow::C_MainWindow(QWidget* parent , const char* name )
 	
 	/* style for non resizable dlgs */
 	int dlg_Style = WStyle_Customize|WStyle_DialogBorder|WStyle_SysMenu|WStyle_Title|WStyle_ContextHelp;
-		
+
+	last_item_ = NULL;
 	dirs_list_ = new C_DirsList(svn_dir_);
 
 	dlg_update_ = new dlg_update(this);
@@ -211,7 +212,7 @@ void C_MainWindow::loadWorkspace(const C_DirPath& path, QString name, bool appen
 		}
 	}
 
-	if (!QDir(path.FullPath()).exists()) {
+	if (path.FullPath().isEmpty() || !QDir(path.FullPath()).exists()) {
 		QMessageBox::information(this, "Open failed", "Workspace/directory does not exist");
 		return;
 	}
@@ -321,8 +322,13 @@ void C_MainWindow::list_dirs_doubleClicked(QListViewItem *item)
     dir.SetPath(workspace_.FullPath());
     dir.SetName(item->text(0));
 
+	C_DirViewItem* dir_item = dynamic_cast<C_DirViewItem*>(item);
+	if (!dir_item) {
+		// should not happen as all dir view items are of type C_DirViewItem
+		return;
+	}
 	QApplication::setOverrideCursor( QCursor(Qt::WaitCursor) );
-	dirs_list_->load(dir, list_dirs, item);
+	dirs_list_->load(dir, list_dirs, dir_item);
 	QApplication::restoreOverrideCursor();
 }
 
@@ -387,22 +393,22 @@ void C_MainWindow::list_files_doubleClicked( QListViewItem *view_item)
 			if (!parent->isOpen()) 
 				list_dirs_doubleClicked(parent);
 
-			QListViewItem *curr_item = parent->firstChild();
-			do {
-                C_DirViewItem *dir_item = dynamic_cast<C_DirViewItem*>(curr_item);
+			QListViewItemIterator iter(parent, QListViewItemIterator::Visible);
+			while (iter.current()) {
+                C_DirViewItem *dir_item = dynamic_cast<C_DirViewItem*>(iter.current());
                 //cout << EsvnLVI::getFullPath(item) << " " <<  curr_item->text(1) << endl;
 
 				if (dir_item && item->FullPath() == dir_item->FullPath()) {
 					emit list_files_selectionChanged(dir_item);
 					//parent->setSelected(false);
 					//curr_item->setSelected(true);
-					list_dirs->setCurrentItem(curr_item);
+					list_dirs->setCurrentItem(dir_item);
 					viewRefresh_localAction_activated();
 
 					return;
 				}
-				curr_item = curr_item->nextSibling();
-			} while (curr_item);
+				++iter;
+			}
 		}
 		else
             show_Log(*item);
@@ -532,7 +538,7 @@ void C_MainWindow::revert(bool is_recursive)
 
     QString text_string ="You are about to revert the following items:\n\n";
     for (size_t i=0; i< selection.size(); i++) 
-        text_string = text_string + selection[i]->Name() + "\n";
+        text_string = text_string + selection[i].Name() + "\n";
 
     QMessageBox mb( "Confirmation",text_string,
         QMessageBox::Question,
@@ -594,7 +600,7 @@ void C_MainWindow::modifyCommitAction_activated() {
 
 	int commit_res = -1;
         if( is_selected_dir_ ) {
-            dlg_commit_->setCommitPath( *sel_path );
+            dlg_commit_->setCommitPath( *sel_path, !dlg_options_->ch_hide_commit->isChecked());
         }
         else {
             getSelectedFiles(selected_files);
@@ -627,15 +633,15 @@ void C_MainWindow::modifyEditAction_activated()
     }
 
     for (PathSelection::const_iterator it = selection.begin(); it != selection.end(); ++it) {
-        const C_Path *path = *it;
-        if (!path->isDir()) {
+        const C_Path& path = *it;
+        if (!path.isDir()) {
             QProcess proc;
-            proc.setWorkingDirectory(path->Path());
+            proc.setWorkingDirectory(path.Path());
             proc.addArgument(editor_set_);
-            proc.addArgument(path->Name());
+            proc.addArgument(path.Name());
             proc.start();
         } else {
-            std::cout << "Cannot edit directory" << path->Name() << std::endl;
+            std::cout << "Cannot edit directory" << path.Name() << std::endl;
         }
     }
 }
@@ -649,11 +655,11 @@ void C_MainWindow::modifyCleanupAction_activated() {
 
     // Better extract directories then call cleanup on list
     for (PathSelection::const_iterator it = selection.begin(); it != selection.end(); ++it) {
-        const C_Path *path = *it;
-        if (path->isDir()) {
-            svn_commands_->cleanup(*path);
+        const C_Path& path = *it;
+        if (path.isDir()) {
+            svn_commands_->cleanup(path);
         } else {
-            std::cout << "Cannot cleanup non-directory" << path->Name() << std::endl;
+            std::cout << "Cannot cleanup non-directory" << path.Name() << std::endl;
         }
     }
 }
@@ -736,19 +742,20 @@ void C_MainWindow::modifyCopy_remote_Action_activated()
 
 	dlg_copy_->lb_url->hide();
 	dlg_copy_->setCaption("Copy (remote)");
-	dlg_copy_->lb_url->setText(selection[0]->Path());
+	dlg_copy_->lb_url->setText(getUrlOnline(selection[0]));
 
 	dlg_copy_->tb_src->setText("U");
 	dlg_copy_->tb_dst->setText("U");
 
     QString sel_names;
     for (size_t i=0; i<selection.size(); i++) {
-        QString url = getUrlOnline(*selection[i]);
+        QString url = getUrlOnline(selection[i]);
         sel_names = sel_names + url;
         if (i != selection.size()-1) 
             sel_names = sel_names + ';';
     }
-    dlg_copy_->le_source->setText(sel_names);
+	dlg_copy_->le_source->setText(sel_names);
+	//dlg_copy_->le_dest->setText(sel_names);
 
 	modifyCopyAction_activated();
 }
@@ -766,14 +773,14 @@ void C_MainWindow::modifyCopy_current_Action_activated()
 
 	dlg_copy_->lb_url->hide();
 	dlg_copy_->setCaption("Copy (local)");
-	dlg_copy_->lb_url->setText(selection[0]->Path());
+	dlg_copy_->lb_url->setText(selection[0].Path());
 
 	dlg_copy_->tb_src->setText("...");
 	dlg_copy_->tb_dst->setText("...");
 
     QString sel_names;
     for (size_t i=0; i<selection.size(); i++) {
-        sel_names = sel_names + selection[i]->Name();
+        sel_names = sel_names + selection[i].Name();
         if (i != selection.size()-1) 
             sel_names = sel_names + ';';
     }
@@ -796,11 +803,13 @@ void C_MainWindow::modifyMoveAction_activated()
     dlg_copy_->setCaption("Move");
 
     // Prepare list of selected names
+	dlg_copy_->tb_src->setText("...");
+	dlg_copy_->tb_dst->setText("...");
 
     dlg_copy_->le_source->setText("");
     QString sel_names;
     for (size_t i=0; i<selection.size(); i++) {
-        sel_names = sel_names + selection[i]->Name();
+        sel_names = sel_names + selection[i].Name();
         if (i != selection.size()-1) 
             sel_names = sel_names + ';';
     }
@@ -816,7 +825,7 @@ void C_MainWindow::modifyMoveAction_activated()
 		QStringList files = QStringList::split(';', dlg_copy_->le_source->text());
 
 		for (size_t i=0; i<files.size(); i++) 
-			svn_commands_->move(selection[0]->Path(), files[i], dlg_copy_->le_dest->text(),dlg_copy_->te_msg->text(), rev); 
+			svn_commands_->move(selection[0].Path(), files[i], dlg_copy_->le_dest->text(),dlg_copy_->te_msg->text(), rev); 
 	}
 }
 
@@ -935,7 +944,14 @@ void C_MainWindow::refreshList()
         sel_path = &workspace_;
     }
 
-	sendToOutput("refresh: " +  sel_path->FullPath() );
+	if (sel_path->FullPath().isEmpty()) {
+		sendToOutput("refresh: invalid path");
+		list_files->clear();
+		auto_refresh_disable_ = false;
+		return;
+	} else {
+		sendToOutput("refresh: " +  sel_path->FullPath() );
+	}
 
 	vector<QString> selected_files;
 	getSelectedFiles(selected_files);
@@ -1123,7 +1139,7 @@ void C_MainWindow::queryViewAction_activated()
         return;
     }
 
-    show_View(*selection[0]);
+    show_View(selection[0]);
 }
 
 void C_MainWindow::queryBlameAction_activated()
@@ -1139,7 +1155,7 @@ void C_MainWindow::queryBlameAction_activated()
 
 	QMessageBox mb( "Confirmation",
 					"You are about to execute \"blame\" command for the following item. \n"
-                    "It may take a while!\n\n" + selection[0]->Name() +"\n",
+                    "It may take a while!\n\n" + selection[0].Name() +"\n",
 					QMessageBox::Question,
 					QMessageBox::Yes | QMessageBox::Default,
 					QMessageBox::No  | QMessageBox::Escape,
@@ -1150,7 +1166,7 @@ void C_MainWindow::queryBlameAction_activated()
 
 
 	QStringList blame;
-	bool res = svn_commands_->blame(*selection[0],"", blame);
+	bool res = svn_commands_->blame(selection[0],"", blame);
 	if (res == 0) {
 		dlg_view_->tb_view->clear();
 		for (size_t i=0; i<blame.size(); i++)
@@ -1204,7 +1220,7 @@ void C_MainWindow::queryDiffAction_activated()
     }
 
     for (size_t i = 0; i < selection.size(); ++i) {
-        const C_Path& current = *selection[i];
+        const C_Path& current = selection[i];
 
         dlg_diff_setup_->lb_url->hide();
         QString url = getUrlOnline(current);
@@ -1256,40 +1272,63 @@ void C_MainWindow::query3_Way_DiffAction_activated()
         return;
     }
 
-	C_SvnEntries svn_entries;
-    C_DirPath currwd;
-    currwd.SetPath(selection[0]->Path());
-	svn_entries.setDir(currwd, false);
-
     for (PathSelection::const_iterator it = selection.begin(); it != selection.end(); ++it)
     {
-        const C_Path *path = *it;
-        if (path->isDir()) {
-            QString text = "Diff3: " + path->Name() + " is a directory";
-            textBrowser->append(text);
-            continue;
-        }
+		const C_Path& path = *it;
+		if (path.isDir()) {
+			QString text = "Diff3: " + path.Name() + " is a directory";
+			textBrowser->append(text);
+			continue;
+		}
 
-        C_SvnFile svn_file = svn_entries.get(path->Name());
+		// Need to load entries for each selected item current directory
+		QString full_path = path.FullPath();
+		C_DirPath file_level;
+
+		// Descend to file current directory
+
+		int pos = full_path.findRev('/');
+
+		if (pos == -1)
+		{
+			file_level.SetPath("");
+			file_level.SetName(full_path);
+		}
+		else
+		{
+			file_level.SetPath(full_path.left(pos));
+			file_level.SetName(full_path.mid(pos+1));
+		}
+
+		// Load entries
+
+		C_SvnEntries svn_entries;
+		C_DirPath currwd;
+		currwd.SetPath(file_level.Path());
+		svn_entries.setDir(currwd, false);
+
+        C_SvnFile svn_file = svn_entries.get(file_level.Name());
 
         if(svn_file.conflict_new_.isNull() || svn_file.conflict_old_.isNull()
             || svn_file.conflict_wrk_.isNull()){
 
-            QString text = "Diff3: " + path->Name() + " is not in conflict";
+			// Display item name as in flat view panel - use original selection
+            QString text = "Diff3: " + path.Name() + " is not in conflict";
             //cout << text << endl;
             textBrowser->append(text);
             continue;
         }
 
         QStringList args;
-        args.push_back(QDir::convertSeparators(qApp->applicationDirPath() + "//esvn-diff-wrapper") );
+        args.push_back(QDir::convertSeparators(qApp->applicationDirPath() + "/esvn-diff-wrapper") );
         args.push_back(dlg_options_->le_diff_3way->text());
         args.push_back(svn_file.conflict_wrk_);
         args.push_back(svn_file.conflict_old_);
         args.push_back(svn_file.conflict_new_);
+		args.push_back(svn_file.name_);
 
         sendToOutput("Diff3: " + svn_file.conflict_wrk_ +  " " + svn_file.conflict_old_ + " " + svn_file.conflict_new_ );
-        svn_commands_->command(currwd.FullPath(), args);
+        svn_commands_->command(file_level.Path(), args);
     }
 }
 
@@ -1369,7 +1408,7 @@ void C_MainWindow::queryProperties_activated()
         return;
     }
 
-	dlg_prop_->setFileDirName(*selection[0]);
+	dlg_prop_->setFileDirName(selection[0]);
 	dlg_prop_->refresh();
 	dlg_prop_->exec();
 }
@@ -1442,7 +1481,8 @@ void C_MainWindow::fileOpen_workspaceAction_activated() {
 	fd->setMode(QFileDialog::DirectoryOnly);
 	if (fd->exec() == QDialog::Accepted) {
 		QString path = fd->selectedFile();
-		path.remove(path.length()-1, 1);
+		if (path.endsWith("/"))
+			path.remove(path.length()-1, 1);
 
 		//cout << path << endl;
 
@@ -1472,7 +1512,7 @@ void C_MainWindow::fileOptions_Action_activated() {
 		is_ext_diff_ = dlg_options_->ch_extdiff->isChecked();
 		dlg_diff_setup_->ch_extdiff->setChecked(is_ext_diff_);
 
-		hide_unknown_ = dlg_options_->ch_hide->isChecked();
+		//hide_unknown_ = dlg_options_->ch_hide->isChecked();
 
 		viewHide_unknownAction->setOn(hide_unknown_);
 		svn_commands_->setUserPassw(dlg_options_->le_user->text(), dlg_options_->le_passw->text());
@@ -1515,7 +1555,7 @@ void C_MainWindow::fileBrowse_Repository_Action_activated()
     getSelectedPaths(selection);
 
     if (!selection.empty()) {
-        dlg_repos_->setReposURL(getUrlOnline(*selection[0]));
+        dlg_repos_->setReposURL(getUrlOnline(selection[0]));
     }
 	dlg_repos_->show();
 }
@@ -1959,6 +1999,10 @@ void C_MainWindow::loadSettings()
 	dlg_options_->le_diff_3way->setText(settings_.readEntry("/progs/diff3", "kdiff3" ));
 	dlg_options_->le_browser->setText(settings_.readEntry("/progs/browser", "mozilla" ));
 
+	bool is_ext_dif = settings_.readBoolEntry("/state/ext_diff", false);
+	dlg_options_->ch_extdiff->setChecked(is_ext_dif);
+	dlg_diff_setup_->ch_extdiff->setChecked(is_ext_dif);
+
 	dlg_options_->ch_extdiff->setChecked(settings_.readBoolEntry("/state/ext_diff", false));
 
 	dlg_options_->rb_cat->setChecked(settings_.readBoolEntry("/state/dblclk_cat", false));
@@ -1969,7 +2013,9 @@ void C_MainWindow::loadSettings()
 
 
 	hide_unknown_ = settings_.readBoolEntry("/state/hide_unknown", false);
-	dlg_options_->ch_hide->setChecked(hide_unknown_);
+    viewHide_unknownAction->setOn(hide_unknown_);
+
+    //dlg_options_->ch_hide->setChecked(hide_unknown_);
 
 	dlg_log_->gr_filters->setHidden(settings_.readBoolEntry("/state/hide_log_filters", false));
 
@@ -2052,7 +2098,7 @@ void C_MainWindow::loadSettings()
 	//busy.hide();
 
 	user_ = dlg_options_->le_user->text();
-	passwd_ = dlg_options_->le_user->text();
+	passwd_ = dlg_options_->le_passw->text();
 
 	dlg_prop_->setParams(svn_commands_, dlg_options_->ch_disable_sprop->isChecked());
 	dlg_workspace_->wsData(&ws_);
@@ -2103,7 +2149,7 @@ QString C_MainWindow::getUrlOnline(const C_Path& localpath) {
 
 	QStringList info;
     PathSelection selection;
-    selection.push_back(&localpath);
+    selection.push_back(localpath);
 	int res = svn_commands_->info(selection, info);
 
 	QString url = "";
@@ -2163,15 +2209,14 @@ int C_MainWindow::getSelectedFiles(vector<QString> &selected_files)
 
     for (PathSelection::const_iterator it = selection.begin(); it != selection.end(); ++it)
     {
-        const C_Path *path = *it;
-        selected_files.push_back(path->Name());
+        const C_Path& path = *it;
+        selected_files.push_back(path.Name());
     }
 
 	return selected_files.size();
 }
 
-template <typename TSelection>
-int getSelection(TSelection& selection, QListView& view)
+int getSelection(PathSelection& selection, QListView& view)
 {
 	selection.clear();
 
@@ -2180,10 +2225,10 @@ int getSelection(TSelection& selection, QListView& view)
     while ( iter.current() ) {
         //QString fname = EsvnLVI::getFullPath(iter.current());
         //cout << "sel: " << fname <<  endl; 
-        typename TSelection::value_type path = dynamic_cast<typename TSelection::value_type>(iter.current());
+        const C_Path* path = dynamic_cast<const C_Path*>(iter.current());
         if (path) {
             //cout << "sel dir: " << path->Path() << " name: " << path->Name() <<  endl; 
-            selection.push_back(path);
+            selection.push_back(*path);
         } else {
             cout << "Warning: no conversion from list view item to path object available" << endl;
         }
@@ -2227,8 +2272,15 @@ int C_MainWindow::getSelectedFiles(PathSelection& selection)
 
 void C_MainWindow::getFilesAndSize(const C_Path& path, vector<QString>& files_and_size)
 {
-	sendToOutput("get files info: " + path.FullPath());
 	files_and_size.clear();
+
+	if (path.FullPath().isEmpty()) {
+		sendToOutput("get files info: invalid path");
+		return;
+	} else {
+		sendToOutput("get files info: " + path.FullPath());
+	}
+
 	QDir files = QDir(path.FullPath());
 	for (size_t i=0; i<files.count(); i++) {
 		if (files[i] != "." && files[i] != "..") {
